#include <winver.h>

#define VER_FILEDESCRIPTION_STR         "Python PCSC Module\0"
#define VER_INTERNALNAME_STR            "_scard.pyd"
#define VER_ORIGINALFILENAME_STR        "_scard.pyd"
#define VER_FILETYPE                    VFT_DLL
#define VER_FILESUBTYPE                 (0)


#include "gemalto.ver"
